#' Convert correlations to distances.
#'
#' Convert a correlation matrix (or other adjacency matrix)
#' into a distance matrix.
#'
#' @param network A numerical matrix (as returned for example by
#' \code{cor}) of correlations between nodes/vertices of a network.
#' @param type One of 'positive' (only positive edges are retained; the default)
#' , 'absolute' (absolute values are retained, so that a negative-weighted edge
#' will have the same distance as if it were positive) or 'scaled' (where
#' negative-weighted edges have greater distances and uncorrelated edges will
#' have intermediate distances).
#' @param range One of 'unit' (edge weights are in the range -1 to 1 as for a
#' correlation matrix; the default) or 'range' (the edge weights will be scaled
#' so that the range of supplied weights ends up on the scale 0 to 1).
#' @return A matrix, of the same dimensions (and row/column names) as the input,
#' containing the edge weights as distances. Thus, the most highly-correlated
#' edges will tend to 0 and uncorrelated edges will tend to 1.
#'
#' @examples
#' \dontrun{
#' require(DataNet)
#' # Load example data and create distance matrix
#' data(hipExpr)
#' network = cor(hipExpr, method="pearson", use="pairwise.complete.obs")
#' distance.matrix = as.distance(network)
#' }
#'
#' @export
as.distance = function(network, type = "positive", range = "unit"){
	max = 1
	min = 0
	if(range == "range"){
		max = max(network)
		min = min(network)
	}
	if(type == "positive"){
		return( max - (network * (network >= 0)) )
	}else if(type == "absolute"){
		return( max - abs(network) )
	}else if(type == "scaled"){
		network = (network - min) / max
		return( 1 - network )
	}else{
		message("The type must be one of 'positive', 'absolute' or 'scaled'.")
	}
}
