#' Convert a distance matrix to an edge list.
#'
#' Convert a distance matrix representing an undirected
#' network into an edge list.
#'
#' This function efficiently transforms the lower triangle of a distance matrix
#' into a \code{data.frame} representing the edges of a network.
#'
#' @param network A numerical matrix (as returned for example by
#' \code{\link{as.distance}}) of distances between nodes/vertices of a network.
#' Values must be positive and will usually lie in the range 0--1 (although
#' no upper bound is set).
#' @param threshold A threshold above which edges will not be included in the
#' edge list.
#' @return A \code{data.frame} representing the edges of a network. The columns
#' \code{source} and \code{target} are character vectors containing the IDs of the
#' source and target nodes respectively (the column and row names of the
#' correponding elements of the distance matrix). The column \code{distance} is
#' a numeric vector containing the value of the of the correponding element of
#' the distance matrix.
#'
#' @examples
#' \dontrun{
#' require(DataNet)
#' # Load example data and create distance matrix
#' data(hipExpr)
#' network = cor(hipExpr, method="pearson", use="pairwise.complete.obs")
#' distance.matrix = as.distance(network)
#'
#' # Convert to edge list
#' edge.list = as.edge.list(distance.matrix, 0.4)
#' }
#'
#' @export
as.edge.list = function(network, threshold){
  n = which(lower.tri(network) & (network <= threshold))
	col = ceiling(n / dim(network)[1])
  row = n - ((col - 1) * dim(network)[1])
  outofbounds = col < 1 | col > dim(network)[2] | row < 1 | row > dim(network)[1]
  col[outofbounds] = NA
  row[outofbounds] = NA
  return(data.frame(source = colnames(network)[col], target = colnames(network)[row], distance = network[n], stringsAsFactors = F))
}
