#' Plot a network as a graph of edges coloured by weight.
#'
#' Plot a network as a graph of edges (vertices are not drawn) coloured by
#' the weight/optimal distance.
#'
#' This function provides a convenient way to plot results of
#' \code{\link{fast.force.layout}}. The edge list, which can be derived from a
#' distance matrix using \code{\link{as.edge.list}}, is the same as provided to
#' \code{\link{fast.force.layout}} which returns the 'coordinates' object also
#' required for this function.
#'
#'
#' @param edge.list A \code{data.frame} with two \code{character} columns,
#' 'source' and 'target', containing the vertex IDs, and one \code{numeric}
#' column, distance, containing the optimal edge distance.
#' @param coordinates A \code{data.frame} with two \code{numeric} columns,
#' \code{x} and \code{y} containing the Cartesian coordinates of the vertices.
#' The vertex IDs (provided as the row names of this \code{data.frame}) must
#' correspond to the vertices used in the edge list. This object can be
#' generated using the function \code{\link{fast.force.layout}}.
#' @param plot.edges A vector, the same length as the number of rows of
#' \code{edge.list}, specifying which edges are to be drawn (default is all
#' edges). The vector may be logical, numeric or character if the edge.list has
#' rownames).
#' @param plot.vertices A vector, the same length as the number of rows of
#' \code{edge.list}, specifying which vertices (nodes) are to be drawn (default
#' is all vertices). The vector may be logical, numeric or character (
#' corresponding to the rownames of \code{coordinates}).
#' @param edge.colours A vector of colours which will be used as a colour scale
#' from the closest to most distant edges. Colours can be specified in any of
#' the three R formats.
#' @param new Logical ( \code{TRUE} by default). Whether to create a new plot.
#' If \code{FALSE}, the graph will be drawn over the existing plot (enabling
#' additional plotting layers to be added).
#' @param return.coordinates Logical ( \code{FALSE} by default). Whether to
#' create a new plot. If \code{TRUE}, the graph will be drawn over the existing
#' plot (enabling additional plotting layers to be added).
#' @param ... Addtional arguments to be passed to \code{\link[graphics]{segments}}.
#' @return If \code{return.coordinates} is \code{TRUE}, then a \code{data.frame}
#' will be returned containing the same information as the \code{edge.list} but
#' with additional columns describing the Cartesian coordinates of the source
#' and target vertices as well as the colour in which the edge was plotted. The
#' default (\code{return.coordinates} = FALSE) returns nothing.
#'
#' @examples
#' \dontrun{
#' require(DataNet)
#' # Load example data and create distance matrix
#' data(hipExpr)
#' network = cor(hipExpr, method="pearson", use="pairwise.complete.obs")
#' distance.matrix = as.distance(network)
#'
#' # Convert to edge list
#' edge.list = as.edge.list(distance.matrix, 0.4)
#'
#' # Calculate coordinates
#' coordinates = fast.force.layout(edge.list)
#'
#' # Plot graph
#' draw.stick.graph(edge.list, coordinates)
#'
#' }
#'
#' @importFrom graphics par plot segments
#' @importFrom grDevices colorRampPalette dev.size
#'
#' @export
draw.stick.graph = function(edge.list, coordinates, plot.edges=1:nrow(edge.list), plot.vertices=1:nrow(coordinates), edge.colours=colorRampPalette(c("red", "yellow", "white"))(256), new=TRUE, return.coordinates=FALSE, ...){
	vertex.coordinates = coordinates[plot.vertices, ]
	if(is.null(rownames(vertex.coordinates))) rownames(vertex.coordinates) = 1:nrow(vertex.coordinates)
	if(is.null(colnames(vertex.coordinates))) colnames(vertex.coordinates) = c("x", "y")

	# Order the edges so strongest are drawn on top (i.e. shortest distance drawn last)
	edge.list = data.frame(source =as.character(edge.list$source), target = as.character(edge.list$target), distance = as.numeric(edge.list$distance), stringsAsFactors = F)
	edge.list = edge.list[plot.edges, ]
	edge.list = edge.list[order(edge.list$distance, decreasing=T), ]

	# Collate coordinates for the vertices comprising each edge
	edge.coordinates = data.frame(
		edge.list,
		source = vertex.coordinates[edge.list$source, ],
		target = vertex.coordinates[edge.list$target, ],
		colour = edge.colours[round(edge.list$distance * (length(edge.colours) - 1)) + 1]
	, stringsAsFactors = F)

	# Scale to fit in plotting area
	# Scaling is base on _all_ coordinates (to allow overplotting of subsets)
	min.x = min(c(coordinates$x, coordinates$x))
	max.x = max(c(coordinates$x, coordinates$x))
	min.y = min(c(coordinates$y, coordinates$y))
	max.y = max(c(coordinates$y, coordinates$y))
	buffer = (max.x - min.x) / dev.size(units="px")[1] # Minimal whitespace on canvas edge (equivalent of 1 px)

	default.mar = par()$mar
	par(mar = c(0, 0, 0, 0))
	if(new) plot(0, xlim=c(min.x-buffer, max.x+buffer), ylim=c(min.y-buffer, max.y+buffer), type="n", xaxs="i", yaxs="i", axes=F, xlab="", ylab="")
	segments(edge.coordinates$source.x, edge.coordinates$source.y, edge.coordinates$target.x, edge.coordinates$target.y, col=edge.coordinates$colour, ...)
	par(mar = default.mar)
	if(return.coordinates) return(edge.coordinates)
}

