#' Force-directed graph layout.
#'
#' Plotting function for network vertices laid out on
#' a grid.
#'
#'
#' @param edge.list A \code{data.frame} representing the edges of a network. The
#' columns \code{source} and \code{target} are character vectors containing the
#' IDs of the source and target nodes respectively. The column \code{distance}
#' is a numeric vector containing the value of the of the correponding element
#' of the distance matrix.
#' @param extend A \code{data.frame} of vertex coordinates, as returned by this
#' function. If supplied, then these coordinates will be used as starting
#' postions. The default value of NULL will place all vertices at the origin.
#' It is important that the random seed is set to be the same for all
#' iterations if a reproducible layout is required. This can be done by calling
#' \code{\link[base]{set.seed}} before performing the layout.
#' @param iterations An integer specifying the number of iterations to be
#' performed. In each iteration, the length of every edge is compared against
#' the desired distance and the position of the associated vertices adjusted
#' accordingly.
#' @param k A \code{double} specifying the fraction of the displacement (the
#' difference between the desired and actual edge lengths) by which each edge
#' length is altered in each itereation. The default value is 0.5 (half the
#' displacement).
#' @param repelling.edges The fraction of edges which are randomly selected in
#' each iteration for calculation of repulsion. Optimally, this value is set at
#' 1 (all edges; default) but for large networks this significantly affects
#' performance. A value of 0.01 (10 % of edges), for example, yields fast
#' computation times while retaining a reasonable layout.
#' @return A \code{data.frame} of vertex coordinates with columns x and y. The
#' vertex IDs are provided as row names.
#'
#' @examples
#' \dontrun{
#' require(DataNet)
#' # Load example data and create distance matrix
#' data(hipExpr)
#' network = cor(hipExpr, method = "pearson", use = "pairwise.complete.obs")
#' distance.matrix = as.distance(network)
#'
#' # Convert to edge list
#' edge.list = as.edge.list(distance.matrix, 0.4)
#'
#' # Calculate coordinates
#' coordinates = fast.force.layout(edge.list, iterations = 100)
#'
#' # Plot graph
#' draw.stick.graph(edge.list, coordinates)
#'
#' }
#'
#' @importFrom stats runif
#'
#' @export
fast.force.layout = function(edge.list, extend = NULL, iterations = 1, k = 0.5, repelling.edges = 1){
	edge.list = data.frame(source =as.character(edge.list$source), target = as.character(edge.list$target), distance = as.numeric(edge.list$distance), stringsAsFactors = F)
	vertices = sort(union(edge.list$source, edge.list$target))
	# All nodes intially placed at the origin
	coordinates = data.frame(x = rep(0, length(vertices)), y = rep(0, length(vertices)))
	if(!is.null(extend)) coordinates = extend
	rownames(coordinates) = vertices

	if(iterations > 0){
		for(iteration in 1:iterations){
			# Calculate current and desired lengths for each edge
			dx = unname(coordinates[edge.list$target, "x"] - coordinates[edge.list$source, "x"]) # x-axis difference between vertex and neighbours
			dy = unname(coordinates[edge.list$target, "y"] - coordinates[edge.list$source, "y"]) # y-axis difference between vertex and neighbours
			l = sqrt(dx ^ 2  + dy ^ 2) # Lengths of existing edges between vertex and neighbours
			# Add repulsion for a random subset of vertices (not all these pairs will be joined by edges)
			repel = matrix(sample(vertices, ceiling(length(vertices) * repelling.edges) * 2, replace = T), ncol = 2)
			rdx = unname(coordinates[repel[, 1], "x"] - coordinates[repel[, 2], "x"])
			rdy = unname(coordinates[repel[, 1], "y"] - coordinates[repel[, 2], "y"])
			rl = sqrt(rdx ^ 2  + rdy ^ 2) # Lengths of existing edges between vertex and neighbours
			too.close = rl < 1 # Only add repulsion to those vertices that are too close
			repel = repel[too.close, , drop = F]
			dx = c(dx, rdx[too.close])
			dy = c(dy, rdy[too.close])
			dl = c((edge.list$distance - l) * k, -rep(k ^ 2, nrow(repel)))
			# Angle detection
			angle = atan2(dx, dy)
			colliders = which(dx==0 & dy==0)
			if(length(colliders > 0)) angle[colliders] = runif(length(angle[colliders])) * 2 * pi - pi # Random angle on collision

			# Calculate amount that the vertices for each edge need to change
			# Source and target move half d relative to each other
			source.dx = by(sin(angle) * dl, c(edge.list$source, repel[, 1]), mean) / 2
			target.dx = by(sin(angle) * dl, c(edge.list$target, repel[, 2]), mean) / 2
			source.dy = by(cos(angle) * dl, c(edge.list$source, repel[, 1]), mean) / 2
			target.dy = by(cos(angle) * dl, c(edge.list$target, repel[, 2]), mean) / 2
			# Adjust each vertex by the centroid of its required changes in each dimension
			coordinates[names(source.dx), "x"] = coordinates[names(source.dx), "x"] - source.dx
			coordinates[names(target.dx), "x"] = coordinates[names(target.dx), "x"] + target.dx
			coordinates[names(source.dy), "y"] = coordinates[names(source.dy), "y"] - source.dy
			coordinates[names(target.dy), "y"] = coordinates[names(target.dy), "y"] + target.dy
		}
	}
	return(as.data.frame(coordinates))
}
